@@ -146,8 +146,8 @@ dbusmenu_server_class_init (DbusmenuServerClass *class)
 	                                         G_SIGNAL_RUN_LAST,
 	                                         G_STRUCT_OFFSET(DbusmenuServerClass, id_update),
 	                                         NULL, NULL,
-	                                         g_cclosure_marshal_VOID__UINT,
-	                                         G_TYPE_NONE, 1, G_TYPE_UINT);
+	                                         g_cclosure_marshal_VOID__INT,
+	                                         G_TYPE_NONE, 1, G_TYPE_INT);
 	/**
 		DbusmenuServer::layout-updated:
 		@arg0: The #DbusmenuServer emitting the signal.
@@ -225,15 +225,22 @@ static void
 set_property (GObject * obj, guint id, const GValue * value, GParamSpec * pspec)
 {
 	DbusmenuServerPrivate * priv = DBUSMENU_SERVER_GET_PRIVATE(obj);
+	GError * error = NULL;
 
 	switch (id) {
 	case PROP_DBUS_OBJECT:
 		g_return_if_fail(priv->dbusobject == NULL);
 		priv->dbusobject = g_value_dup_string(value);
-		DBusGConnection * connection = dbus_g_bus_get(DBUS_BUS_SESSION, NULL);
-		dbus_g_connection_register_g_object(connection,
-											priv->dbusobject,
-											obj);
+		DBusGConnection * connection = dbus_g_bus_get(DBUS_BUS_SESSION, &error);
+
+		if (connection == NULL || error != NULL) {
+			g_warning("Unable to get session bus: %s", error == NULL ? "No message" : error->message);
+			if (error != NULL) { g_error_free(error); }
+		} else {
+			dbus_g_connection_register_g_object(connection,
+			                                    priv->dbusobject,
+			                                    obj);
+		}
 		break;
 	case PROP_ROOT_NODE:
 		if (priv->root != NULL) {
@@ -404,6 +411,16 @@ _dbusmenu_server_get_layout (DbusmenuServer * server, gint parent, guint * revis
 		}
 	} else {
 		DbusmenuMenuitem * item = dbusmenu_menuitem_find_id(priv->root, parent);
+		if (item == NULL) {
+			if (error != NULL) {
+				g_set_error(error,
+				            error_quark(),
+				            INVALID_MENUITEM_ID,
+				            "The ID supplied %d does not refer to a menu item we have",
+				            parent);
+			}
+			return FALSE;
+		}
 		dbusmenu_menuitem_buildxml(item, xmlarray);
 	}
 	g_ptr_array_add(xmlarray, NULL);
